Class: SPARQL::Client
- Inherits:
-
Object
- Object
- SPARQL::Client
- Defined in:
- lib/sparql/client.rb,
lib/sparql/client/query.rb,
lib/sparql/client/update.rb,
lib/sparql/client/version.rb,
lib/sparql/client/repository.rb
Overview
A SPARQL 1.0/1.1 client for RDF.rb.
Defined Under Namespace
Modules: Update, VERSION Classes: ClientError, MalformedQuery, Query, QueryElement, Repository, ServerError
Constant Summary collapse
- RESULT_JSON =
'application/sparql-results+json'.freeze
- RESULT_XML =
'application/sparql-results+xml'.freeze
- RESULT_CSV =
'text/csv'.freeze
- RESULT_TSV =
'text/tab-separated-values'.freeze
- RESULT_BOOL =
Sesame-specific
'text/boolean'.freeze
- RESULT_BRTR =
Sesame-specific
'application/x-binary-rdf-results-table'.freeze
- RESULT_ALL =
[ RESULT_JSON, RESULT_XML, RESULT_BOOL, "#{RESULT_TSV};q=0.8", "#{RESULT_CSV};q=0.2", '*/*;q=0.1' ].join(', ').freeze
- GRAPH_ALL =
( RDF::Format.content_types.keys + ['*/*;q=0.1'] ).join(', ').freeze
- ACCEPT_JSON =
{'Accept' => RESULT_JSON}.freeze
- ACCEPT_XML =
{'Accept' => RESULT_XML}.freeze
- ACCEPT_CSV =
{'Accept' => RESULT_CSV}.freeze
- ACCEPT_TSV =
{'Accept' => RESULT_TSV}.freeze
- ACCEPT_BRTR =
{'Accept' => RESULT_BRTR}.freeze
- ACCEPT_RESULTS =
{'Accept' => RESULT_ALL}.freeze
- ACCEPT_GRAPH =
{'Accept' => GRAPH_ALL}.freeze
- DEFAULT_PROTOCOL =
1.0- DEFAULT_METHOD =
:post- XMLNS =
{'sparql' => 'http://www.w3.org/2005/sparql-results#'}.freeze
Instance Attribute Summary collapse
-
#headers ⇒ Hash{String => String}
readonly
The HTTP headers that will be sent in requests to the endpoint.
-
#options ⇒ Hash{Symbol => Object}
readonly
Any miscellaneous configuration.
-
#url ⇒ RDF::URI, RDF::Queryable
readonly
The SPARQL endpoint URL, or an RDF::Queryable instance, to use the native SPARQL engine.
Class Method Summary collapse
-
.finalize(klass) ⇒ Object
Close the http connection when object is deallocated.
- .parse_csv_bindings(csv, nodes = {}) ⇒ <RDF::Query::Solutions>
- .parse_json_bindings(json, nodes = {}) ⇒ <RDF::Query::Solutions>
- .parse_json_value(value, nodes = {}) ⇒ RDF::Value
- .parse_tsv_bindings(tsv, nodes = {}) ⇒ <RDF::Query::Solutions>
- .parse_xml_bindings(xml, nodes = {}) ⇒ <RDF::Query::Solutions>
- .parse_xml_value(value, nodes = {}) ⇒ RDF::Value
-
.serialize_patterns(patterns, use_vars = false) ⇒ String
Serializes a SPARQL graph.
-
.serialize_predicate(value, rdepth = 0) ⇒ String
Serializes a SPARQL predicate.
-
.serialize_uri(uri) ⇒ String
Serializes a URI or URI string into SPARQL syntax.
-
.serialize_value(value, use_vars = false) ⇒ String
Serializes an
RDF::Valueinto SPARQL syntax.
Instance Method Summary collapse
-
#ask(*args, **options) ⇒ Query
Executes a boolean
ASKquery. - #call_query_method(meth, *args, **options) ⇒ Object
-
#clear(what, *arguments) ⇒ Object
Executes a
CLEARoperation. -
#clear_graph(graph_uri, **options) ⇒ void
Executes a ‘CLEAR GRAPH` operation.
-
#close ⇒ void
Closes a client instance by finishing the connection.
-
#construct(*args, **options) ⇒ Query
Executes a graph
CONSTRUCTquery. -
#delete_data(data, **options) ⇒ void
Executes a ‘DELETE DATA` operation.
-
#delete_insert(delete_graph, insert_graph = nil, where_graph = nil, **options) ⇒ void
Executes a
DELETE/INSERToperation. -
#describe(*args, **options) ⇒ Query
Executes a
DESCRIBEquery. -
#initialize(url, **options, &block) ⇒ Client
constructor
Initialize a new sparql client, either using the URL of a SPARQL endpoint or an
RDF::Queryableinstance to use the native SPARQL gem. -
#insert_data(data, **options) ⇒ void
Executes an ‘INSERT DATA` operation.
-
#inspect ⇒ String
Returns a developer-friendly representation of this object.
-
#inspect! ⇒ void
Outputs a developer-friendly representation of this object to
stderr. -
#nodes ⇒ Object
Returns a mapping of blank node results for this client.
- #parse_rdf_serialization(response, **options) ⇒ RDF::Enumerable
- #parse_response(response, **options) ⇒ Object
-
#query(query, **options) ⇒ Array<RDF::Query::Solution>
Executes a SPARQL query and returns the parsed results.
-
#response(query, **options) ⇒ String
Executes a SPARQL query and returns the Net::HTTP::Response of the result.
-
#select(*args, **options) ⇒ Query
Executes a tuple
SELECTquery. -
#update(query, **options) ⇒ void
Executes a SPARQL update operation.
Constructor Details
#initialize(url, **options, &block) ⇒ Client
Initialize a new sparql client, either using the URL of a SPARQL endpoint or an RDF::Queryable instance to use the native SPARQL gem.
95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 |
# File 'lib/sparql/client.rb', line 95 def initialize(url, **, &block) case url when RDF::Queryable @url, @options = url, .dup else @url, @options = RDF::URI.new(url.to_s), .dup @headers = @options.delete(:headers) || {} @http = http_klass(@url.scheme) # Close the http connection when object is deallocated ObjectSpace.define_finalizer(self, self.class.finalize(@http)) end if block_given? case block.arity when 1 then block.call(self) else instance_eval(&block) end end end |
Instance Attribute Details
#headers ⇒ Hash{String => String} (readonly)
The HTTP headers that will be sent in requests to the endpoint.
70 71 72 |
# File 'lib/sparql/client.rb', line 70 def headers @headers end |
#options ⇒ Hash{Symbol => Object} (readonly)
Any miscellaneous configuration.
76 77 78 |
# File 'lib/sparql/client.rb', line 76 def @options end |
#url ⇒ RDF::URI, RDF::Queryable (readonly)
The SPARQL endpoint URL, or an RDF::Queryable instance, to use the native SPARQL engine.
64 65 66 |
# File 'lib/sparql/client.rb', line 64 def url @url end |
Class Method Details
.finalize(klass) ⇒ Object
Close the http connection when object is deallocated
117 118 119 120 121 |
# File 'lib/sparql/client.rb', line 117 def self.finalize(klass) proc do klass.shutdown if klass.respond_to?(:shutdown) end end |
.parse_csv_bindings(csv, nodes = {}) ⇒ <RDF::Query::Solutions>
453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 |
# File 'lib/sparql/client.rb', line 453 def self.parse_csv_bindings(csv, nodes = {}) require 'csv' unless defined?(::CSV) csv = CSV.parse(csv.to_s) unless csv.is_a?(Array) vars = csv.shift solutions = RDF::Query::Solutions.new csv.each do |row| solution = RDF::Query::Solution.new row.each_with_index do |v, i| term = case v when /^_:(.*)$/ then nodes[$1] ||= RDF::Node($1) when /^\w+:.*$/ then RDF::URI(v) else RDF::Literal(v) end solution[vars[i].to_sym] = term end solutions << solution end solutions end |
.parse_json_bindings(json, nodes = {}) ⇒ <RDF::Query::Solutions>
413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 |
# File 'lib/sparql/client.rb', line 413 def self.parse_json_bindings(json, nodes = {}) require 'json' unless defined?(::JSON) json = JSON.parse(json.to_s) unless json.is_a?(Hash) case when json.has_key?('boolean') json['boolean'] when json.has_key?('results') solutions = json['results']['bindings'].map do |row| row = row.inject({}) do |cols, (name, value)| cols.merge(name.to_sym => parse_json_value(value, nodes)) end RDF::Query::Solution.new(row) end RDF::Query::Solutions.new(solutions) end end |
.parse_json_value(value, nodes = {}) ⇒ RDF::Value
435 436 437 438 439 440 441 442 443 444 445 446 447 |
# File 'lib/sparql/client.rb', line 435 def self.parse_json_value(value, nodes = {}) case value['type'].to_sym when :bnode nodes[id = value['value']] ||= RDF::Node.new(id) when :uri RDF::URI.new(value['value']) when :literal RDF::Literal.new(value['value'], datatype: value['datatype'], language: value['xml:lang']) when :'typed-literal' RDF::Literal.new(value['value'], datatype: value['datatype']) else nil end end |
.parse_tsv_bindings(tsv, nodes = {}) ⇒ <RDF::Query::Solutions>
477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 |
# File 'lib/sparql/client.rb', line 477 def self.parse_tsv_bindings(tsv, nodes = {}) tsv = tsv.lines.map {|l| l.chomp.split("\t")} unless tsv.is_a?(Array) vars = tsv.shift.map {|h| h.sub(/^\?/, '')} solutions = RDF::Query::Solutions.new tsv.each do |row| solution = RDF::Query::Solution.new row.each_with_index do |v, i| if !v.empty? term = RDF::NTriples.unserialize(v) || case v when /^\d+\.\d*[eE][+-]?[0-9]+$/ then RDF::Literal::Double.new(v) when /^\d*\.\d+[eE][+-]?[0-9]+$/ then RDF::Literal::Double.new(v) when /^\d*\.\d+$/ then RDF::Literal::Decimal.new(v) when /^\d+$/ then RDF::Literal::Integer.new(v) else RDF::Literal(v) end nodes[term.id] = term if term.is_a? RDF::Node solution[vars[i].to_sym] = term end end solutions << solution end solutions end |
.parse_xml_bindings(xml, nodes = {}) ⇒ <RDF::Query::Solutions>
506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 |
# File 'lib/sparql/client.rb', line 506 def self.parse_xml_bindings(xml, nodes = {}) xml.force_encoding(::Encoding::UTF_8) if xml.respond_to?(:force_encoding) if defined?(::Nokogiri) xml = Nokogiri::XML(xml).root unless xml.is_a?(Nokogiri::XML::Document) case when boolean = xml.xpath("//sparql:boolean", XMLNS)[0] boolean.text == 'true' when results = xml.xpath("//sparql:results", XMLNS)[0] solutions = results.elements.map do |result| row = {} result.elements.each do |binding| name = binding.attr('name').to_sym value = binding.elements.first row[name] = parse_xml_value(value, nodes) end RDF::Query::Solution.new(row) end RDF::Query::Solutions.new(solutions) end else # REXML xml = REXML::Document.new(xml).root unless xml.is_a?(REXML::Element) case when boolean = xml.elements['boolean'] boolean.text == 'true' when results = xml.elements['results'] solutions = results.elements.map do |result| row = {} result.elements.each do |binding| name = binding.attributes['name'].to_sym value = binding.select { |node| node.kind_of?(::REXML::Element) }.first row[name] = parse_xml_value(value, nodes) end RDF::Query::Solution.new(row) end RDF::Query::Solutions.new(solutions) end end end |
.parse_xml_value(value, nodes = {}) ⇒ RDF::Value
551 552 553 554 555 556 557 558 559 560 561 562 563 |
# File 'lib/sparql/client.rb', line 551 def self.parse_xml_value(value, nodes = {}) case value.name.to_sym when :bnode nodes[id = value.text] ||= RDF::Node.new(id) when :uri RDF::URI.new(value.text) when :literal lang = value.respond_to?(:attr) ? value.attr('xml:lang') : value.attributes['xml:lang'] datatype = value.respond_to?(:attr) ? value.attr('datatype') : value.attributes['datatype'] RDF::Literal.new(value.text, language: lang, datatype: datatype) else nil end end |
.serialize_patterns(patterns, use_vars = false) ⇒ String
Serializes a SPARQL graph
638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 |
# File 'lib/sparql/client.rb', line 638 def self.serialize_patterns(patterns, use_vars = false) patterns.map do |pattern| serialized_pattern = case pattern when SPARQL::Client::QueryElement then [pattern.to_s] else RDF::Statement.from(pattern).to_triple.each_with_index.map do |v, i| if i == 1 SPARQL::Client.serialize_predicate(v) else SPARQL::Client.serialize_value(v, use_vars) end end end serialized_pattern.join(' ') + ' .' end end |
.serialize_predicate(value, rdepth = 0) ⇒ String
Serializes a SPARQL predicate
617 618 619 620 621 622 623 624 625 626 627 628 629 |
# File 'lib/sparql/client.rb', line 617 def self.serialize_predicate(value,rdepth=0) case value when nil RDF::Query::Variable.new.to_s when String then value when Array s = value.map{|v|serialize_predicate(v,rdepth+1)}.join rdepth > 0 ? "(#{s})" : s when RDF::Value # abbreviate RDF.type in the predicate position per SPARQL grammar value.equal?(RDF.type) ? 'a' : serialize_value(value) end end |
.serialize_uri(uri) ⇒ String
Serializes a URI or URI string into SPARQL syntax.
584 585 586 587 588 589 590 |
# File 'lib/sparql/client.rb', line 584 def self.serialize_uri(uri) case uri when String then RDF::NTriples.serialize(RDF::URI(uri)) when RDF::URI then RDF::NTriples.serialize(uri) else raise ArgumentError, "expected the graph URI to be a String or RDF::URI, but got #{uri.inspect}" end end |
.serialize_value(value, use_vars = false) ⇒ String
Serializes an RDF::Value into SPARQL syntax.
599 600 601 602 603 604 605 606 607 608 |
# File 'lib/sparql/client.rb', line 599 def self.serialize_value(value, use_vars = false) # SPARQL queries are UTF-8, but support ASCII-style Unicode escapes, so # the N-Triples serializer is fine unless it's a variable: case when value.nil? then RDF::Query::Variable.new.to_s when value.variable? then value.to_s when value.node? then (use_vars ? RDF::Query::Variable.new(value.id) : value) else RDF::NTriples.serialize(value) end end |
Instance Method Details
#ask(*args, **options) ⇒ Query
Executes a boolean ASK query.
138 139 140 |
# File 'lib/sparql/client.rb', line 138 def ask(*args, **) call_query_method(:ask, *args, **) end |
#call_query_method(meth, *args, **options) ⇒ Object
291 292 293 294 295 296 297 298 |
# File 'lib/sparql/client.rb', line 291 def call_query_method(meth, *args, **) client = self result = Query.send(meth, *args, **) (class << result; self; end).send(:define_method, :execute) do client.query(self) end result end |
#clear(what, *arguments) ⇒ void #clear(what, *arguments, **options) ⇒ void
Executes a CLEAR operation.
This requires that the endpoint support SPARQL 1.1 Update.
285 286 287 |
# File 'lib/sparql/client.rb', line 285 def clear(what, *arguments) self.update(Update::Clear.new(what, *arguments)) end |
#clear_graph(graph_uri, **options) ⇒ void
This method returns an undefined value.
Executes a ‘CLEAR GRAPH` operation.
This is a convenience wrapper for the #clear method.
250 251 252 |
# File 'lib/sparql/client.rb', line 250 def clear_graph(graph_uri, **) self.clear(:graph, graph_uri, **) end |
#close ⇒ void
This method returns an undefined value.
Closes a client instance by finishing the connection. The client is unavailable for any further data operations; an IOError is raised if such an attempt is made. I/O streams are automatically closed when they are claimed by the garbage collector.
127 128 129 130 131 |
# File 'lib/sparql/client.rb', line 127 def close @http.shutdown if @http @http = nil self end |
#construct(*args, **options) ⇒ Query
Executes a graph CONSTRUCT query.
165 166 167 |
# File 'lib/sparql/client.rb', line 165 def construct(*args, **) call_query_method(:construct, *args, **) end |
#delete_data(data, **options) ⇒ void
This method returns an undefined value.
Executes a ‘DELETE DATA` operation.
This requires that the endpoint support SPARQL 1.1 Update.
217 218 219 |
# File 'lib/sparql/client.rb', line 217 def delete_data(data, **) self.update(Update::DeleteData.new(data, **)) end |
#delete_insert(delete_graph, insert_graph = nil, where_graph = nil, **options) ⇒ void
This method returns an undefined value.
Executes a DELETE/INSERT operation.
This requires that the endpoint support SPARQL 1.1 Update.
233 234 235 |
# File 'lib/sparql/client.rb', line 233 def delete_insert(delete_graph, insert_graph = nil, where_graph = nil, **) self.update(Update::DeleteInsert.new(delete_graph, insert_graph, where_graph, **)) end |
#describe(*args, **options) ⇒ Query
Executes a DESCRIBE query.
156 157 158 |
# File 'lib/sparql/client.rb', line 156 def describe(*args, **) call_query_method(:describe, *args, **) end |
#insert_data(data, **options) ⇒ void
This method returns an undefined value.
Executes an ‘INSERT DATA` operation.
This requires that the endpoint support SPARQL 1.1 Update.
Note that for inserting non-trivial amounts of data, you probably ought to consider using the RDF store’s native bulk-loading facilities or APIs, as ‘INSERT DATA` operations entail comparably higher parsing overhead.
196 197 198 |
# File 'lib/sparql/client.rb', line 196 def insert_data(data, **) self.update(Update::InsertData.new(data, **)) end |
#inspect ⇒ String
Returns a developer-friendly representation of this object.
667 668 669 |
# File 'lib/sparql/client.rb', line 667 def inspect sprintf("#<%s:%#0x(%s)>", self.class.name, __id__, url.to_s) end |
#inspect! ⇒ void
This method returns an undefined value.
Outputs a developer-friendly representation of this object to stderr.
659 660 661 |
# File 'lib/sparql/client.rb', line 659 def inspect! warn(inspect) end |
#nodes ⇒ Object
Returns a mapping of blank node results for this client.
304 305 306 |
# File 'lib/sparql/client.rb', line 304 def nodes @nodes ||= {} end |
#parse_rdf_serialization(response, **options) ⇒ RDF::Enumerable
569 570 571 572 573 574 575 576 |
# File 'lib/sparql/client.rb', line 569 def parse_rdf_serialization(response, **) = {content_type: response.content_type} unless [:content_type] if reader = RDF::Reader.for() reader.new(response.body) else raise RDF::ReaderError, "no RDF reader was found for #{}." end end |
#parse_response(response, **options) ⇒ Object
390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 |
# File 'lib/sparql/client.rb', line 390 def parse_response(response, **) case [:content_type] || response.content_type when NilClass response.body when RESULT_BOOL # Sesame-specific response.body == 'true' when RESULT_JSON self.class.parse_json_bindings(response.body, nodes) when RESULT_XML self.class.parse_xml_bindings(response.body, nodes) when RESULT_CSV self.class.parse_csv_bindings(response.body, nodes) when RESULT_TSV self.class.parse_tsv_bindings(response.body, nodes) else parse_rdf_serialization(response, **) end end |
#query(query, **options) ⇒ Array<RDF::Query::Solution>
Executes a SPARQL query and returns the parsed results.
318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 |
# File 'lib/sparql/client.rb', line 318 def query(query, **) @op = :query @alt_endpoint = [:endpoint] case @url when RDF::Queryable require 'sparql' unless defined?(::SPARQL::Grammar) begin SPARQL.execute(query, @url, optimize: true, **) rescue SPARQL::MalformedQuery $stderr.puts "error running #{query}: #{$!}" raise end else parse_response(response(query, **), **) end end |
#response(query, **options) ⇒ String
Executes a SPARQL query and returns the Net::HTTP::Response of the result.
369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 |
# File 'lib/sparql/client.rb', line 369 def response(query, **) headers = [:headers] || @headers headers['Accept'] = [:content_type] if [:content_type] request(query, headers) do |response| case response when Net::HTTPBadRequest # 400 Bad Request raise MalformedQuery.new(response.body + " Processing query #{query}") when Net::HTTPClientError # 4xx raise ClientError.new(response.body + " Processing query #{query}") when Net::HTTPServerError # 5xx raise ServerError.new(response.body + " Processing query #{query}") when Net::HTTPSuccess # 2xx response end end end |
#select(*args, **options) ⇒ Query
Executes a tuple SELECT query.
147 148 149 |
# File 'lib/sparql/client.rb', line 147 def select(*args, **) call_query_method(:select, *args, **) end |
#update(query, **options) ⇒ void
This method returns an undefined value.
Executes a SPARQL update operation.
346 347 348 349 350 351 352 353 354 355 356 357 |
# File 'lib/sparql/client.rb', line 346 def update(query, **) @op = :update @alt_endpoint = [:endpoint] case @url when RDF::Queryable require 'sparql' unless defined?(::SPARQL::Grammar) SPARQL.execute(query, @url, update: true, optimize: true, **) else response(query, **) end self end |